"""
info.py

This module exposes the project information from the pyproject.toml file.

Usage:

    from util.info import PROJECT_NAME, PROJECT_VERSION, PROJECT_DESCRIPTION
"""

import tomllib

from sapiumai_common_tools.logger import setup_logger

logger = setup_logger(name=__name__)
try:
    with open(file="pyproject.toml", mode="rb") as f:
        data = tomllib.load(f)

    # Project information
    PROJECT_NAME: str = data.get("project", {}).get("name", "")
    PROJECT_VERSION: str = data.get("project", {}).get("version", "")
    PROJECT_DESCRIPTION: str = data.get("project", {}).get("description", "")
    if not all([PROJECT_NAME, PROJECT_VERSION, PROJECT_DESCRIPTION]):
        logger.warning(
            msg="Project information not found in pyproject.toml."
            "Using defaults."
        )
        PROJECT_NAME = "Unknown"
        PROJECT_VERSION = "Unknown"
        PROJECT_DESCRIPTION = "Unknown"
except FileNotFoundError:
    logger.warning(
        msg="pyproject.toml not found. Using defaults for project"
        " information."
    )
    PROJECT_NAME = "Unknown"
    PROJECT_VERSION = "Unknown"
    PROJECT_DESCRIPTION = "Unknown"



"""
logger.py

Creates a logger object that logs to a file.

Usage:
    from util.logger import setup_logger

    logger = setup_logger(name=__name__)
"""

from logging import DEBUG, FileHandler, Formatter, Logger, getLogger
from os import mkdir, path

LOG_DIR = "./logs"
LOG_FILE = "log.txt"

if not path.exists(path=LOG_DIR):
    mkdir(path=LOG_DIR)


def setup_logger(name: str, level: int = DEBUG) -> Logger:
    """
    Creates a logger object that logs to a file.

    Attributes:
        name (str): The name of the logger.
        level (int): The level of the logger.
    """
    logger: Logger = getLogger(name=name)
    logger.setLevel(level=level)

    # Create file handler which logs messages
    file_handler = FileHandler(filename=path.join(LOG_DIR, LOG_FILE))
    file_handler.setLevel(level=DEBUG)
    # Create formatter and add it to the handler
    formatter = Formatter(
        fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    file_handler.setFormatter(fmt=formatter)

    # Add the handlers to the logger
    if (
        not logger.handlers
    ):  # To avoid adding handlers multiple times in some environments
        logger.addHandler(hdlr=file_handler)
    return logger



import json
from os import mkdir, path, remove, rename

from sapiumai_common_tools.info import PROJECT_NAME, PROJECT_VERSION


class TestInfo:

    def test_project_name(self):
        assert PROJECT_NAME == "sapiumai_common_tools"

    def test_project_version(self):
        # update this when you update the version in pyproject.toml
        assert PROJECT_VERSION == "0.1.1"


class TestLogger:
    def setup_method(self):
        if path.exists(path="./logs/log.txt"):
            remove(path="./logs/log.txt")

    def test_log_exists(self):
        from sapiumai_common_tools.logger import setup_logger

        logger = setup_logger(name="TestLogger")
        logger.info(msg="Test log")

        assert path.exists(path="./logs/log.txt")

    def teardown_method(self):
        if path.exists(path="./logs/log.txt"):
            remove(path="./logs/log.txt")


class TestConfig:

    def setup_class(self):
        if not path.exists(path="./config"):
            mkdir(path="./config")
        if path.exists(path="./config/config.json"):
            rename(src="./config/config.json", dst="./config/config.json.bak")

        sample_data = {"project": "sapium-agent", "env": "test"}
        with open(file="./config/config.json", mode="w") as f:
            json.dump(obj=sample_data, fp=f)

    def teardown_class(self):
        if path.exists(path="./config/config.json"):
            remove(path="./config/config.json")

        if path.exists(path="./config/config.json.bak"):
            rename(src="./config/config.json.bak", dst="./config/config.json")

    def test_config_exists(self):
        from sapiumai_common_tools.config import config

        assert config != {}
        assert config["project"] == "sapium-agent"
        assert config["env"] == "test"



"""
config.py

This module provides functionality to load the application configuration
from a JSON file located in the './config' directory. It defines:

- load_config: Reads 'config.json' and returns its contents as a NestedConfig.
- config: A module-level variable holding the loaded configuration.

If the configuration file is missing or cannot be read, an empty config
dictionary is returned and an error is logged.
"""

import json
from os import path
from typing import Union

from sapiumai_common_tools.logger import setup_logger

logger = setup_logger(name=__name__)


Primitive = Union[str, int, float, bool]
NestedConfig = dict[str, dict[str, Primitive]]


def load_config() -> NestedConfig:
    """
    Loads the config file.

    Returns:
        NestedConfig: The config file as a nested dictionary.
    """
    CONFIG_PATH = "./config"
    try:
        filename = path.join(CONFIG_PATH, "config.json")
        with open(file=filename, mode="r") as f:
            return json.load(fp=f)
    except FileNotFoundError as e:
        logger.error(msg=f"Error loading config: {e}")
        return {}


config: NestedConfig = load_config()
